using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace NetShips {
    /// <summary>
    /// Spravuje vechny lod v hracm poli.
    /// </summary>
    public class ShipsCollection //: ICollection<Lo>
    {
        private FieldPoint[,] gameField;
        private Ship[] shipsList = new Ship[0];

        /// <summary>
        /// Konstruktor tdy LodCollection
        /// </summary>
        /// <param name="HracPlocha">Dvou rozmrn pole Bod nad kterm se budou lod vytvet.</param>     
        internal ShipsCollection(ref FieldPoint[,] HracPlocha) {
            gameField = HracPlocha;

        }

        /// <summary>
        /// Vyvol refresh vech lod.
        /// </summary>
        public void Refresh() {
            foreach(Ship lo in shipsList) {
                lo.Refresh();
            }
        }


        #region ICollection<Lo> Members

        /// <summary>
        /// Pid do kolekce lod novou instanci lod.
        /// </summary>
        /// <param name="TypLod">Typ lod k vytvoen.</param>
        /// <param name="pozice">Pozice na kterou novou lo umstit.</param>
        /// <param name="hel">hel otoen Lodi.</param>
        public Ship Add(Ship.ShipTypes_ENUM TypLod, System.Drawing.Point pozice, Ship.Directions_ENUM hel) {
            Ship newLo;

            // vytvoen nov instance lod, odchytvna vjimka, kdy se lo nevleze do pole.
            try {
                newLo = new Ship(TypLod, pozice, gameField, hel);
            }
            catch(ArgumentException ex) {
                throw ex;
            }

            newLo.Colection = this;

            // rozen seznamu lod
            System.Array.Resize<Ship>(ref shipsList, shipsList.Length + 1);
            shipsList[shipsList.Length - 1] = newLo;

            return newLo;

        }
        
        /// <summary>
        /// Pid do kolekce lod novou instanci lod.
        /// </summary>
        /// <param name="TypLod">Typ lod k vytvoen.</param>
        /// <param name="pozice">Pozice na kterou novou lo umstit.</param>
        public Ship Add(Ship.ShipTypes_ENUM TypLod, System.Drawing.Point pozice) {
            return Add(TypLod, pozice, Ship.Directions_ENUM.ANGLE_0);
        }

        /// <summary>
        /// Pid do kolekce lod novou instanci lod.
        /// </summary>
        /// <param name="TypLod">Typ lod k vytvoen.</param>
        /// <returns>Vrac odkaz na nov vytvoenou Lo.</returns>
        public Ship Add(Ship.ShipTypes_ENUM TypLod) {
            return Add(TypLod, new Point());
        }


        /// <summary>
        /// Zru ze seznamu vechny instance lod.
        /// </summary>
        public void Clear() {
            for(int i = 0; i < shipsList.Length; i++) {
                shipsList[i].Dispose();
                shipsList[i] = null;
            }
            shipsList = null;
        }


        /// <summary>
        /// Zjist jestli zadan lo je soust pole.
        /// </summary>
        /// <param name="item">Lo kter se bude hledat v poli.</param>
        /// <returns>Vrac True, pokud byla lo v poli nalezena.</returns>
        public bool Contains(Ship item) {
            for(int i = 0; i < shipsList.Length; i++)
                if(shipsList[i].Equals(item))
                    return true;
            return false;
        }


        /// <summary>
        /// Vrac poet vytvoench lod.
        /// </summary>
        public int Count {
            get {
                if(shipsList == null)
                    return 0;
                else
                    return shipsList.Length;
            }
        }

        /// <summary>
        /// Poet nesestelench lod.
        /// </summary>
        public int CountNotDown {
            get {
                int i = 0;
                foreach(Ship l in this)
                    if(l.State > 0)
                        i++;
                return i;
            }
        }

        /// <summary>
        /// Vrt odkaz na lo obsazujc vybran bod.
        /// </summary>
        /// <param name="bod">Bod, kter budu zkoumat.</param>
        /// <returns>Vrt odkaz na instanci Lodi, pokud Bod pedstavuje Lo (a u s chybou, nebo bez chyby), nebo null, pokud je bod Voda.</returns>
        public Ship Get_Reference(FieldPoint bod) {
            if(bod.Type == FieldPoint.FieldPointType_ENUM.Ship || bod.Type == FieldPoint.FieldPointType_ENUM.Error) {
                foreach(Ship lo in shipsList)
                    foreach(FieldPoint bodLodi in lo.Points)
                        if(object.ReferenceEquals(bodLodi, bod))
                            return lo;
            }
            return null;
        }

        /// <summary>
        /// Z kolekce odebere lo zadanou parametrem.
        /// </summary>
        /// <param name="item">Lo, kter se m odebrat.</param>
        /// <returns>Vrac True, pokud se odebrn zdailo, False pokud neusplo = odebran objekt nebyl nalezen.</returns>
        public bool Remove(Ship item) {
            Boolean Nalezen = false;
            for(int i = 0; i < shipsList.Length; i++) {
                if(shipsList[i].Equals(item) || Nalezen) {
                    if(!Nalezen)
                        shipsList[i].Dispose();

                    Nalezen = true;
                    if(i < shipsList.Length - 1)
                        shipsList[i] = shipsList[i + 1];
                    else {
                        Array.Resize<Ship>(ref shipsList, shipsList.Length - 1);
                        return true;
                    }
                }
            }
            return false;
        }

        #endregion

        public MyEnumerator GetEnumerator() {
            return new MyEnumerator(this);
        }

        public class MyEnumerator {
            int nIndex;
            ShipsCollection collection;
            public MyEnumerator(ShipsCollection coll) {
                collection = coll;
                nIndex = -1;
            }

            public bool MoveNext() {
                nIndex++;
                return (nIndex < collection.shipsList.GetLength(0));
            }

            public Ship Current {
                get {
                    return (collection.shipsList[nIndex]);
                }
            }
        }

    }
}
